<?php

namespace Drupal\bulk_update_user_roles\Form;

use Drupal\bulk_update_user_roles\Batch\BulkUpdateUserRoles;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\RoleStorageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for bulk updating user roles.
 */
class BulkUpdateUserForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The role storage.
   *
   * @var \Drupal\user\RoleStorageInterface
   */
  protected $roleStorage;

  /**
   * Constructs a new BulkUpdateUserForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\user\RoleStorageInterface $role_storage
   *   The role storage.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RoleStorageInterface $role_storage) {
    $this->entityTypeManager = $entity_type_manager;
    $this->roleStorage = $role_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_type.manager')->getStorage('user_role')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bulk_update_user_roles_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $update_options = [
      'add' => $this->t('Assign roles'),
      'remove' => $this->t('Remove roles'),
    ];

    $roles = $this->roleStorage->loadMultiple();
    $role_options = [];
    foreach ($roles as $role) {
      if (!in_array($role->id(), ['anonymous', 'authenticated'])) {
        $role_options[$role->id()] = $role->label();
      }
    }

    $user_storage = $this->entityTypeManager->getStorage('user');
    $query = $user_storage->getQuery()
      ->condition('uid', 1, '>')
      ->accessCheck(FALSE);
    $user_ids = $query->execute();

    $user_count = count($user_ids);

    $form['note'] = [
      '#markup' => '<strong>' . $this->t('NOTE: Bulk Update User Roles will exclude uid 1 from updates.') . '</strong>',
    ];

    $form['available_users'] = [
      '#markup' => '<strong>' . $this->t('Available users: @count', ['@count' => $user_count]) . '</strong>',
    ];

    $form['select_all_users'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Select all users'),
      '#default_value' => FALSE,
      '#description' => $this->t('Check to select all users.'),
    ];

    $form['selected_users'] = [
      '#type' => 'select',
      '#title' => $this->t('Select users to update'),
      '#options' => $this->getUserOptions($user_ids),
      '#multiple' => TRUE,
      '#size' => 10,
      '#default_value' => [],
      '#states' => [
        'visible' => [
          ':input[name="select_all_users"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $form['buur_update_option'] = [
      '#type' => 'select',
      '#title' => $this->t('Update options'),
      '#options' => $update_options,
      '#default_value' => NULL,
      '#required' => TRUE,
    ];

    $form['buur_user_roles'] = [
      '#type' => 'select',
      '#title' => $this->t('Role(s) to assign/remove'),
      '#options' => $role_options,
      '#default_value' => [],
      '#required' => TRUE,
      '#multiple' => TRUE,
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * Helper function to get user options.
   */
  protected function getUserOptions(array $user_ids) {
    $users = $this->entityTypeManager->getStorage('user')->loadMultiple($user_ids);
    $options = [];
    foreach ($users as $user) {
      $options[$user->id()] = $user->getDisplayName();
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $roles_selected = $form_state->getValue('buur_user_roles');
    $select_all_users = $form_state->getValue('select_all_users');
    $user_storage = $this->entityTypeManager->getStorage('user');
    $query = $user_storage->getQuery()
      ->condition('uid', 1, '>')
      ->accessCheck(FALSE);

    if (!$select_all_users) {
      $user_ids = $form_state->getValue('selected_users');
    }
    else {
      $user_ids = $query->execute();
    }

    if (!empty($user_ids)) {
      $batch = [];
      $operation = $form_state->getValue('buur_update_option') === 'add'
        ? [BulkUpdateUserRoles::class, 'addRolesToUser']
        : [BulkUpdateUserRoles::class, 'removeRolesFromUser'];

      foreach ($user_ids as $uid) {
        $batch['operations'][] = [
          $operation, [$uid, $roles_selected],
        ];
      }

      $batch['finished'] = [BulkUpdateUserRoles::class, 'finished'];
      $batch['title'] = $form_state->getValue('buur_update_option') === 'add'
        ? $this->t('Processing add users roles batch')
        : $this->t('Processing remove users roles batch');
      $batch['init_message'] = $this->t('Users roles batch starting.');
      $batch['progress_message'] = $this->t('Processed @current out of @total.');
      $batch['error_message'] = $this->t('Batch process has encountered an error.');

      batch_set($batch);
    }
    else {
      $this->messenger()->addMessage($this->t('No users to update the roles.'));
    }
  }

}
